/***************************************************************************
 * This confidential and proprietary software may be used only as
 * authorised by a licensing agreement from ARM Limited
 *  (C) COPYRIGHT 2001 ARM Limited
 *      ALL RIGHTS RESERVED
 * The entire notice above must be reproduced on all authorised
 * copies and copies may only be made to the extent permitted
 * by a licensing agreement from ARM Limited.
 ***************************************************************************/
/***************************************************************************
 *								      
 *	$Id: logic.c,v 1.2 1999/11/22 10:58:54 mwelsh Exp $
 *
 ***************************************************************************/
 

#include <stdio.h>
#include "logic.h"
#include "platform.h"

int main(void)
{
  int i,j, SwitchValue;

  /*** conf mem controller ***/
  ConfigureMemory();

  /*** check that LM0 is present ***/
  if(!(word_read(INTEGRATOR_SC_DEC) & 0x10)) {
    printf("ERROR: There is no expansion header fitted\n");
    return 1;
  }

  /*** turn off all LEDS ***/
  word_write(LM_LEDS,0x1FF);

  /*** print out some system information ***/
  printf("Headers present   0x%02X\n",word_read(INTEGRATOR_SC_DEC) & 0xFF);
  printf("System controller ID=0x%X\n",word_read(INTEGRATOR_SC_BASE));
  printf("Core Module       ID=0x%X\n",word_read(INTEGRATOR_HDR_BASE));

  TestSsram(LMSSRAMSTART,LMSSRAMEND);

  /*** set up LM clock frequencies ***/
  word_write(LM_LOCK,UNLOCK);
  word_write(LM_OSC1,FIVEMHZ);
  word_write(LM_OSC2,TENMHZ);
  word_write(LM_LOCK,LOCK);

  /*** flash the LEDs ***/    
  j=0;
  printf("Flashing LEDS...\n");  
  while(j<30) {
    for (i=0;i<100000;i++);
      word_write(LM_LEDS,0x1AA);
    for (i=0;i<100000;i++);
      word_write(LM_LEDS,0x155);
    j++;
  }
  
  /*** Mirror the switches to the LEDs ***/       
  printf("Mirroring switch settings to LEDs...\n");    
  while(1) {  
    SwitchValue = word_read(LM_SW);
    word_write(LM_LEDS,(SwitchValue & 0xFF) + 0x100);  //add 0x100 to keep RED LED off
  }
  
  return 0;  //never executed
}

 
/************************************************
 * Configure memory controller
 ************************************************/

void ConfigureMemory(void)
{
  int data, dimm_size, temp1, temp2, dimm_data, hdr_no, no_hdrs, dimm_present;

  /*** determine what processor we are in the stack ***/
  hdr_no = word_read(INTEGRATOR_HDR_STAT) & 0xF;

  /*** determine how many headers there are in the stack ***/
  data = word_read(INTEGRATOR_SC_DEC);
  switch (data & 0xF) {
    case 1:
      no_hdrs = 1; break;
    case 3:
      no_hdrs = 2; break;
    case 7:
      no_hdrs = 3; break;
    case 0xf:
      no_hdrs = 4; break;
    default:
      printf("ERROR reading INTEGRATOR_SC_DEC.\n"); 
      exit(1);      
  }

  /*** determine dimm size ***/
  data = word_read(INTEGRATOR_HDR_SPDBASE + 0x1C);
  temp1 = (data >> 24) & 0xFF;
  data = word_read(INTEGRATOR_HDR_SPDBASE + 0x4);
  temp2 = (data >> 8) & 0xFF;
  dimm_size = temp1 * temp2 * 4;

  /*** set up memory controller for the dimm ***/
  dimm_present = 1;
  switch(dimm_size) {
    case 16:
      dimm_data = 0x0;  break;
    case 32:
      dimm_data = 0x1;  break;
    case 64:
      dimm_data = 0x2;  break;
    case 128:
      dimm_data = 0x3;  break;
    case 256:
      dimm_data = 0x4;  break;
    default:
      dimm_present = 0; break;
  }

  if(dimm_present) {
    data = word_read(INTEGRATOR_HDR_SDRAM);
    word_write(INTEGRATOR_HDR_SDRAM,((data & 0xFFFFFFC3) | (dimm_data << 2)));
    printf("DIMM fitted %dMB\n",dimm_size);     
  }
  else {
    dimm_size = 0;
    printf("You do not have a DIMM fitted, ensure that $top_of_memory\n");
    printf("is set to 0x40000 each time the debugger is started.\n\n"); 
  }
  printf("This is processor %d in a stack of %d\n",hdr_no,no_hdrs); 
}


/*************************************************************
 * misc functions 
 *************************************************************/

int TestSsram(int start, int end)
{
  int addr, dataw, datar;
  
  /*** in all tests the data written to each word is <address + 0x12345678> ***/
  printf("\nTesting SSRAM...\n");
  
  /*** write bytes, read back in words ***/
  if(ClearSsram(LMSSRAMSTART,LMSSRAMEND))
    printf("ERROR: SSRAM clear failed\n");
  for (addr=start;addr<=end;addr+=4) {
    dataw = addr + 0x12345678;
    byte_write(addr,(dataw & 0xFF));
    byte_write(addr+1,((dataw>>8) & 0xFF));
    byte_write(addr+2,((dataw>>16) & 0xFF));
    byte_write(addr+3,((dataw>>24) & 0xFF));                  
    datar = word_read(addr);
    if(datar != dataw) {
      word_write(LM_LEDS,0xFF); //light RED LED to indicate failure    
      printf("\nERROR: SSRAM byte write or word read failed\n");
      exit(1);
    }
  }  
  printf("\nPASS SSRAM byte write and word read\n");
     
  
  /*** write half words, read back in bytes ***/
  if(ClearSsram(LMSSRAMSTART,LMSSRAMEND))
    printf("SSRAM clear failed\n");
  for (addr=start;addr<=end;addr+=4) {
    dataw = addr + 0x12345678;
    hword_write(addr,(dataw & 0xFFFF));
    hword_write(addr+2,((dataw>>16) & 0xFFFF));
    datar= (byte_read(addr) & 0xFF) +
           ((byte_read(addr+1) & 0xFF) << 8) +
           ((byte_read(addr+2) & 0xFF) << 16) +
           ((byte_read(addr+3) & 0xFF) << 24);
    if(datar != dataw) {
      word_write(LM_LEDS,0xFF); //light RED LED to indicate failure    
      printf("\nERROR: SSRAM half word write or byte read failed\n");
      exit(1);
    }
  }  
  printf("PASS SSRAM half word write and byte read\n");

  
  /*** write words, read back in half words ***/
  if(ClearSsram(LMSSRAMSTART,LMSSRAMEND))
    printf("SSRAM clear failed\n");
  for (addr=start;addr<=end;addr+=4) {
    dataw = addr + 0x12345678;
    word_write(addr,dataw);
    datar = (hword_read(addr) & 0xFFFF) + ((hword_read(addr+2) & 0xFFFF) << 16);
    if(datar != dataw) {
      word_write(LM_LEDS,0xFF); //light RED LED to indicate failure    
      printf("\nERROR: SSRAM word write or half word read failed\n");
      exit(1);
    }
  }  
  printf("PASS SSRAM word write and half word read\n\n");
  
  return 0;
}


int ClearSsram(int start, int end)
{
  int addr;
  
  for (addr=start;addr<=end;addr+=4) {
    word_write(addr,0);
    if (word_read(addr) != 0) {
      word_write(LM_LEDS,0xFF); //light RED LED to indicate failure
      exit(1);
    }  
  }
  return 0;           //success
}
